@@ -1,33 +1,56 @@
 # -*- coding: utf-8 -*-
 import os, time
-from functools import wraps
 from .cross import pickle, md5hex
 
-from django.conf import settings
+from funcy import wraps
 
-from .conf import redis_client, handle_connection_failure
+from .conf import settings
 from .utils import func_cache_key, cached_view_fab
+from .redis import redis_client, handle_connection_failure
 
 
-__all__ = ('cache', 'cached', 'cached_view', 'file_cache', 'CacheMiss')
+__all__ = ('cache', 'cached', 'cached_view', 'file_cache', 'CacheMiss', 'FileCache', 'RedisCache')
 
 
 class CacheMiss(Exception):
     pass
 
+class CacheKey(str):
+    @classmethod
+    def make(cls, value, cache=None, timeout=None):
+        self = CacheKey(value)
+        self.cache = cache
+        self.timeout = timeout
+        return self
+
+    def get(self):
+        self.cache.get(self)
+
+    def set(self, value):
+        self.cache.set(self, value, self.timeout)
+
+    def delete(self):
+        self.cache.delete(self)
 
 class BaseCache(object):
     """
     Simple cache with time-based invalidation
     """
-    def _cached(self, timeout=None, extra=None, _get_key=None):
+    def cached(self, timeout=None, extra=None, key_func=func_cache_key):
         """
         A decorator for caching function calls
         """
+        # Support @cached (without parentheses) form
+        if callable(timeout):
+            return self.cached(key_func=key_func)(timeout)
+
         def decorator(func):
             @wraps(func)
             def wrapper(*args, **kwargs):
-                cache_key = 'c:' + _get_key(func, args, kwargs, extra)
+                if not settings.CACHEOPS_ENABLED:
+                    return func(*args, **kwargs)
+
+                cache_key = 'c:' + key_func(func, args, kwargs, extra)
                 try:
                     result = self.get(cache_key)
                 except CacheMiss:
@@ -37,18 +60,20 @@ class BaseCache(object):
                 return result
 
             def invalidate(*args, **kwargs):
-                cache_key = 'c:' + _get_key(func, args, kwargs, extra)
+                cache_key = 'c:' + key_func(func, args, kwargs, extra)
                 self.delete(cache_key)
             wrapper.invalidate = invalidate
 
+            def key(*args, **kwargs):
+                cache_key = 'c:' + key_func(func, args, kwargs, extra)
+                return CacheKey.make(cache_key, cache=self, timeout=timeout)
+            wrapper.key = key
+
             return wrapper
         return decorator
 
-    def cached(self, timeout=None, extra=None):
-        return self._cached(timeout, extra, _get_key=func_cache_key)
-
     def cached_view(self, timeout=None, extra=None):
-        return cached_view_fab(self._cached)(timeout, extra)
+        return cached_view_fab(self.cached)(timeout=timeout, extra=extra)
 
 
 class RedisCache(BaseCache):
@@ -78,16 +103,13 @@ cached = cache.cached
 cached_view = cache.cached_view
 
 
-FILE_CACHE_DIR = getattr(settings, 'FILE_CACHE_DIR', '/tmp/cacheops_file_cache')
-FILE_CACHE_TIMEOUT = getattr(settings, 'FILE_CACHE_TIMEOUT', 60*60*24*30)
-
 class FileCache(BaseCache):
     """
     A file cache which fixes bugs and misdesign in django default one.
     Uses mtimes in the future to designate expire time. This makes unnecessary
     reading stale files.
     """
-    def __init__(self, path, timeout=FILE_CACHE_TIMEOUT):
+    def __init__(self, path, timeout=settings.FILE_CACHE_TIMEOUT):
         self._dir = path
         self._default_timeout = timeout
 
@@ -143,4 +165,4 @@ class FileCache(BaseCache):
         except (IOError, OSError):
             pass
 
-file_cache = FileCache(FILE_CACHE_DIR)
+file_cache = FileCache(settings.FILE_CACHE_DIR)
