@@ -1,18 +1,15 @@
 # -*- coding: utf-8 -*-
 import re
-from functools import wraps
 import json
 import inspect
-import threading
-import six
-from funcy import memoize
+from funcy import memoize, compose, wraps, any, partial
+from funcy.py2 import mapcat
 from .cross import md5hex
 
-import django
 from django.db import models
 from django.http import HttpRequest
 
-from .conf import redis_client
+from .conf import model_profile
 
 
 # NOTE: we don't serialize this fields since their values could be very long
@@ -20,11 +17,11 @@ from .conf import redis_client
 NOT_SERIALIZED_FIELDS = (
     models.FileField,
     models.TextField, # One should not filter by long text equality
+    models.BinaryField,
 )
-if hasattr(models, 'BinaryField'):
-    NOT_SERIALIZED_FIELDS += (models.BinaryField,)
 
 
+@memoize
 def non_proxy(model):
     while model._meta.proxy:
         # Every proxy model has exactly one non abstract parent model
@@ -32,13 +29,21 @@ def non_proxy(model):
                        if issubclass(b, models.Model) and not b._meta.abstract)
     return model
 
+def model_family(model):
+    """
+    Returns a list of all proxy models, including subclasess, superclassses and siblings.
+    """
+    def class_tree(cls):
+        return [cls] + mapcat(class_tree, cls.__subclasses__())
 
-if django.VERSION < (1, 6):
-    def get_model_name(model):
-        return model._meta.module_name
-else:
-    def get_model_name(model):
-        return model._meta.model_name
+    # NOTE: we also list multitable submodels here, we just don't care.
+    #       Cacheops doesn't support them anyway.
+    return class_tree(non_proxy(model))
+
+
+@memoize
+def family_has_profile(cls):
+    return any(model_profile, model_family(cls))
 
 
 class MonkeyProxy(object):
@@ -64,18 +69,14 @@ def monkey_mix(cls, mixin, methods=None):
     cls._no_monkey = MonkeyProxy(cls)
 
     if methods is None:
-        # NOTE: there no such thing as unbound method in Python 3, it uses naked functions,
-        #       so we use some six based altering here
-        isboundmethod = inspect.isfunction if six.PY3 else inspect.ismethod
-        methods = inspect.getmembers(mixin, isboundmethod)
+        methods = [(name, m) for name, m in mixin.__dict__.items() if inspect.isfunction(m)]
     else:
-        methods = [(m, getattr(mixin, m)) for m in methods]
+        methods = [(m, mixin.__dict__[m]) for m in methods]
 
     for name, method in methods:
         if hasattr(cls, name):
             setattr(cls._no_monkey, name, getattr(cls, name))
-        # NOTE: remember, there is no bound methods in Python 3
-        setattr(cls, name, six.get_unbound_function(method))
+        setattr(cls, name, method)
 
 
 @memoize
@@ -89,26 +90,48 @@ def stamp_fields(model):
 
 ### Cache keys calculation
 
-def func_cache_key(func, args, kwargs, extra=None):
+def func_cache_key(func, args, kwargs, extra=None, debug=False):
     """
     Calculate cache key based on func and arguments
     """
-    factors = [func.__module__, func.__name__, func.__code__.co_firstlineno, args, kwargs, extra]
-    return md5hex(json.dumps(factors, sort_keys=True, default=str))
+    def obj_key(obj):
+        if isinstance(obj, models.Model):
+            return '%s.%s.%s' % (obj._meta.app_label, obj._meta.model_name, obj.pk)
+        elif inspect.isfunction(obj):
+            factors = [obj.__module__, obj.__name__]
+            # Really useful to ignore this when editing code
+            if not debug and hasattr(func, '__code__'):
+                factors.append(obj.__code__.co_firstlineno)
+            return factors
+        else:
+            return str(obj)
+
+    factors = [obj_key(func), args, kwargs, extra]
+    return md5hex(json.dumps(factors, sort_keys=True, default=obj_key))
+
+debug_cache_key = partial(func_cache_key, debug=True)
 
 def view_cache_key(func, args, kwargs, extra=None):
     """
     Calculate cache key for view func.
     Use url instead of not properly serializable request argument.
     """
-    uri = args[0].build_absolute_uri()
+    if hasattr(args[0], 'build_absolute_uri'):
+        uri = args[0].build_absolute_uri()
+    else:
+        uri = args[0]
     return 'v:' + func_cache_key(func, args[1:], kwargs, extra=(uri, extra))
 
 def cached_view_fab(_cached):
+    def force_render(response):
+        if hasattr(response, 'render') and callable(response.render):
+            response.render()
+        return response
+
     def cached_view(*dargs, **dkwargs):
         def decorator(func):
-            dkwargs['_get_key'] = view_cache_key
-            cached_func = _cached(*dargs, **dkwargs)(func)
+            dkwargs['key_func'] = view_cache_key
+            cached_func = _cached(*dargs, **dkwargs)(compose(force_render, func))
 
             @wraps(func)
             def wrapper(request, *args, **kwargs):
@@ -118,28 +141,16 @@ def cached_view_fab(_cached):
                     return func(request, *args, **kwargs)
 
                 return cached_func(request, *args, **kwargs)
+
+            if hasattr(cached_func, 'invalidate'):
+                wrapper.invalidate = cached_func.invalidate
+                wrapper.key = cached_func.key
+
             return wrapper
         return decorator
     return cached_view
 
 
-### Lua script loader
-
-import os.path
-
-STRIP_RE = re.compile(r'TOSTRIP.*/TOSTRIP', re.S)
-
-@memoize
-def load_script(name, strip=False):
-    # TODO: strip comments
-    filename = os.path.join(os.path.dirname(__file__), 'lua/%s.lua' % name)
-    with open(filename) as f:
-        code = f.read()
-    if strip:
-        code = STRIP_RE.sub('', code)
-    return redis_client.register_script(code)
-
-
 ### Whitespace handling for template tags
 
 from django.utils.safestring import mark_safe
@@ -151,9 +162,3 @@ def carefully_strip_whitespace(text):
     text = re.sub(r'>\s*\n\s*<', NEWLINE_BETWEEN_TAGS, text)
     text = re.sub(r'>\s{2,}<', SPACE_BETWEEN_TAGS, text)
     return text
-
-
-# This will help mimic thread globals via dicts
-
-def get_thread_id():
-    return threading.current_thread().ident
